from prowler.providers.gcp.lib.audit_info.models import GCP_Audit_Info

gcp_audit_info = GCP_Audit_Info(
    credentials=None,
    project_id=None,
    audit_resources=None,
    audit_metadata=None,
)
